@@ -15,7 +15,12 @@ from vyper.exceptions import (
     ZeroDivisionException,
 )
 from vyper.settings import VYPER_ERROR_CONTEXT_LINES, VYPER_ERROR_LINE_NUMBERS
-from vyper.utils import MAX_DECIMAL_PLACES, SizeLimits, annotate_source_code
+from vyper.utils import (
+    MAX_DECIMAL_PLACES,
+    SizeLimits,
+    annotate_source_code,
+    checksum_encode,
+)
 
 NODE_BASE_ATTRIBUTES = (
     "_children",
@@ -23,6 +28,7 @@ NODE_BASE_ATTRIBUTES = (
     "_parent",
     "ast_type",
     "node_id",
+    "_metadata",
 )
 NODE_SRC_ATTRIBUTES = (
     "col_offset",
@@ -122,11 +128,18 @@ def compare_nodes(left_node: "VyperNode", right_node: "VyperNode") -> bool:
     return True
 
 
-def _to_node(value, parent):
-    # if value is a Python node or dict representing a node, convert to a Vyper node
-    if isinstance(value, (dict, python_ast.AST)):
-        return get_node(value, parent)
-    return value
+def _to_node(obj, parent):
+    # if object is a Python node or dict representing a node, convert to a Vyper node
+    if isinstance(obj, (dict, python_ast.AST)):
+        return get_node(obj, parent)
+    if isinstance(obj, VyperNode):
+        # if object is already a vyper node, make sure the parent is set correctly
+        # and fix any missing source offsets
+        obj._parent = parent
+        for field_name in NODE_SRC_ATTRIBUTES:
+            if getattr(obj, field_name) is None:
+                setattr(obj, field_name, getattr(parent, field_name, None))
+    return obj
 
 
 def _to_dict(value):
@@ -233,6 +246,7 @@ class VyperNode:
         self._parent = parent
         self._depth = getattr(parent, "_depth", -1) + 1
         self._children: set = set()
+        self._metadata: dict = {}
 
         for field_name in NODE_SRC_ATTRIBUTES:
             # when a source offset is not available, use the parent's source offset
@@ -600,6 +614,38 @@ class Module(TopLevel):
         new_node._depth = old_node._depth
         parent._children.add(new_node)
 
+    def add_to_body(self, node: VyperNode) -> None:
+        """
+        Add a new node to the body of this node.
+
+        This method should be used in favor of directly modifying `body`, as
+        it also sets the parent/child relationships used in node traversal.
+
+        Arguments
+        ---------
+        node: VyperNode
+            Vyper node to be appended to the body of the this node.
+        """
+        self.body.append(node)
+        node._depth = self._depth + 1
+        node._parent = self
+        self._children.add(node)
+
+    def remove_from_body(self, node: VyperNode) -> None:
+        """
+        Remove a node from the body of this node.
+
+        This method should be used in favor of directly modifying `body`, as
+        it also removes the parent/child relationship used in node traversal.
+
+        Arguments
+        ---------
+        node: VyperNode
+            Vyper node to be appended to the body of the this node.
+        """
+        self.body.remove(node)
+        self._children.remove(node)
+
 
 class FunctionDef(TopLevel):
     __slots__ = ("args", "returns", "decorator_list", "pos")
@@ -712,7 +758,7 @@ class Decimal(Num):
         super().validate()
 
 
-class Hex(Num):
+class Hex(Constant):
     """
     A hexadecimal value, e.g. `0xFF`
 
@@ -723,10 +769,17 @@ class Hex(Num):
     """
 
     __slots__ = ()
+    _translated_fields = {"n": "value"}
 
     def validate(self):
         if len(self.value) % 2:
             raise InvalidLiteral("Hex notation requires an even number of digits", self)
+        if len(self.value) == 42 and checksum_encode(self.value) != self.value:
+            raise InvalidLiteral(
+                "Address checksum mismatch. If you are sure this is the right "
+                f"address, the correct checksummed form is: {checksum_encode(self.value)}",
+                self,
+            )
 
 
 class Str(Constant):
@@ -751,11 +804,13 @@ class Bytes(Constant):
     def __init__(self, parent: Optional["VyperNode"] = None, **kwargs: dict):
         super().__init__(parent, **kwargs)
         if isinstance(self.value, str):
-            self.value = self.value.encode("utf8")
+            # convert hex string to bytes
+            length = len(self.value) // 2 - 1
+            self.value = int(self.value, 16).to_bytes(length, "big")
 
     def to_dict(self):
         ast_dict = super().to_dict()
-        ast_dict["value"] = self.value.decode("utf8")
+        ast_dict["value"] = f"0x{self.value.hex()}"
         return ast_dict
 
     @property
@@ -786,10 +841,7 @@ class NameConstant(Constant):
 
 
 class Name(VyperNode):
-    __slots__ = (
-        "id",
-        "_type",
-    )
+    __slots__ = ("id",)
 
 
 class Expr(VyperNode):
@@ -1078,6 +1130,14 @@ class In(VyperNode):
         return left in right
 
 
+class NotIn(VyperNode):
+    __slots__ = ()
+    _description = "exclusion"
+
+    def _op(self, left, right):
+        return left not in right
+
+
 class Call(VyperNode):
     __slots__ = ("func", "args", "keywords", "keyword")
 
