@@ -1,6 +1,6 @@
 import warnings
 from collections import OrderedDict
-from typing import Any, Dict, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple
 
 from vyper import ast as vy_ast
 from vyper.ast.validation import validate_call_args
@@ -26,6 +26,7 @@ from vyper.exceptions import (
     StateAccessViolation,
     StructureException,
 )
+from vyper.utils import keccak256
 
 
 class FunctionVisibility(StringEnum):
@@ -70,9 +71,11 @@ class ContractFunction(BaseTypeDefinition):
         The name of the function.
     arguments : OrderedDict
         Function input arguments as {'name': BaseType}
-    arg_count : Tuple[int, int] | int
-        The number of input arguments. If given as a tuple, the value represents
-        (min, max) when default values are given.
+    min_arg_count : int
+        The minimum number of required input arguments.
+    max_arg_count : int
+        The maximum number of required input arguments. When a function has no
+        default arguments, this value is the same as `min_arg_count`.
     kwarg_keys : List
         List of optional input argument keys.
     function_visibility : FunctionVisibility
@@ -89,7 +92,8 @@ class ContractFunction(BaseTypeDefinition):
         self,
         name: str,
         arguments: OrderedDict,
-        arg_count: Union[Tuple[int, int], int],
+        min_arg_count: int,
+        max_arg_count: int,
         return_type: Optional[BaseTypeDefinition],
         function_visibility: FunctionVisibility,
         state_mutability: StateMutability,
@@ -105,11 +109,12 @@ class ContractFunction(BaseTypeDefinition):
         )
         self.name = name
         self.arguments = arguments
-        self.arg_count = arg_count
+        self.min_arg_count = min_arg_count
+        self.max_arg_count = max_arg_count
         self.return_type = return_type
         self.kwarg_keys = []
-        if isinstance(arg_count, tuple):
-            self.kwarg_keys = list(self.arguments)[arg_count[0] :]  # noqa: E203
+        if min_arg_count < max_arg_count:
+            self.kwarg_keys = list(self.arguments)[min_arg_count:]  # noqa: E203
         self.visibility = function_visibility
         self.mutability = state_mutability
         self.nonreentrant = nonreentrant
@@ -153,6 +158,7 @@ class ContractFunction(BaseTypeDefinition):
             abi["name"],
             arguments,
             len(arguments),
+            len(arguments),
             return_type,
             function_visibility=FunctionVisibility.EXTERNAL,
             state_mutability=StateMutability.from_abi(abi),
@@ -257,20 +263,36 @@ class ContractFunction(BaseTypeDefinition):
                 f"Visibility must be set to one of: {', '.join(FunctionVisibility.values())}", node
             )
 
+        if node.name == "__default__":
+            if kwargs["function_visibility"] != FunctionVisibility.EXTERNAL:
+                raise FunctionDeclarationException(
+                    "Default function must be marked as `@external`", node
+                )
+            if node.args.args:
+                raise FunctionDeclarationException(
+                    "Default function may not receive any arguments", node.args.args[0]
+                )
+
         if "state_mutability" not in kwargs:
             # Assume nonpayable if not set at all (cannot accept Ether, but can modify state)
             kwargs["state_mutability"] = StateMutability.NONPAYABLE
 
+        if (
+            kwargs["state_mutability"] in (StateMutability.VIEW, StateMutability.PURE)
+            and "nonreentrant" in kwargs
+        ):
+            raise StructureException("Cannot use reentrancy guard on view or pure functions", node)
+
         # call arguments
-        arg_count: Union[Tuple[int, int], int] = len(node.args.args)
-        if node.args.defaults:
-            arg_count = (
-                len(node.args.args) - len(node.args.defaults),
-                len(node.args.args),
+        if node.args.defaults and node.name == "__init__":
+            raise FunctionDeclarationException(
+                "Constructor may not use default arguments", node.args.defaults[0]
             )
 
         arguments = OrderedDict()
-        defaults = [None] * (len(node.args.args) - len(node.args.defaults)) + node.args.defaults
+        max_arg_count = len(node.args.args)
+        min_arg_count = max_arg_count - len(node.args.defaults)
+        defaults = [None] * min_arg_count + node.args.defaults
 
         namespace = get_namespace()
         for arg, value in zip(node.args.args, defaults):
@@ -280,8 +302,6 @@ class ContractFunction(BaseTypeDefinition):
                 )
             if arg.arg in arguments:
                 raise ArgumentException(f"Function contains multiple inputs named {arg.arg}", arg)
-            if arg.arg in namespace["self"].members:
-                raise NamespaceCollision("Name shadows an existing storage-scoped value", arg)
             if arg.arg in namespace:
                 raise NamespaceCollision(arg.arg, arg)
 
@@ -320,7 +340,7 @@ class ContractFunction(BaseTypeDefinition):
         else:
             raise InvalidType("Function return value must be a type name or tuple", node.returns)
 
-        return cls(node.name, arguments, arg_count, return_type, **kwargs)
+        return cls(node.name, arguments, min_arg_count, max_arg_count, return_type, **kwargs)
 
     @classmethod
     def from_AnnAssign(cls, node: vy_ast.AnnAssign) -> "ContractFunction":
@@ -349,11 +369,43 @@ class ContractFunction(BaseTypeDefinition):
             node.target.id,
             args_dict,
             len(arguments),
+            len(arguments),
             return_type,
             function_visibility=FunctionVisibility.EXTERNAL,
             state_mutability=StateMutability.NONPAYABLE,
         )
 
+    @property
+    def method_ids(self) -> Dict[str, int]:
+        """
+        Dict of `{signature: four byte selector}` for this function.
+
+        * For functions without default arguments the dict contains one item.
+        * For functions with default arguments, there is one key for each
+          function signfature.
+        """
+        arg_types = [i.canonical_type for i in self.arguments.values()]
+
+        if not self.has_default_args:
+            return _generate_method_id(self.name, arg_types)
+
+        method_ids = {}
+        for i in range(self.min_arg_count, self.max_arg_count + 1):
+            method_ids.update(_generate_method_id(self.name, arg_types[:i]))
+        return method_ids
+
+    @property
+    def is_constructor(self) -> bool:
+        return self.name == "__init__"
+
+    @property
+    def is_fallback(self) -> bool:
+        return self.name == "__default__"
+
+    @property
+    def has_default_args(self) -> bool:
+        return self.min_arg_count < self.max_arg_count
+
     def get_signature(self) -> Tuple[Tuple, Optional[BaseTypeDefinition]]:
         return tuple(self.arguments.values()), self.return_type
 
@@ -365,7 +417,7 @@ class ContractFunction(BaseTypeDefinition):
         kwarg_keys = self.kwarg_keys.copy()
         if node.get("func.value.id") != "self":
             kwarg_keys += ["gas", "value"]
-        validate_call_args(node, self.arg_count, kwarg_keys)
+        validate_call_args(node, (self.min_arg_count, self.max_arg_count), kwarg_keys)
 
         if self.mutability < StateMutability.PAYABLE:
             kwarg_node = next((k for k in node.keywords if k.arg == "value"), None)
@@ -382,3 +434,54 @@ class ContractFunction(BaseTypeDefinition):
                 validate_expected_type(kwarg.arg, kwarg.value)
 
         return self.return_type
+
+    def to_abi_dict(self) -> List[Dict]:
+        abi_dict: Dict = {"stateMutability": self.mutability.value}
+
+        if self.is_fallback:
+            abi_dict["type"] = "fallback"
+            return [abi_dict]
+
+        if self.is_constructor:
+            abi_dict["type"] = "constructor"
+        else:
+            abi_dict["type"] = "function"
+            abi_dict["name"] = self.name
+
+        abi_dict["inputs"] = [_generate_abi_type(v, k) for k, v in self.arguments.items()]
+
+        typ = self.return_type
+        if typ is None:
+            abi_dict["outputs"] = []
+        elif isinstance(typ, TupleDefinition):
+            abi_dict["outputs"] = [_generate_abi_type(i) for i in typ.value_type]  # type: ignore
+        elif isinstance(typ, StructDefinition):
+            abi_dict["outputs"] = [_generate_abi_type(v, k) for k, v in typ.members.items()]
+        else:
+            abi_dict["outputs"] = [_generate_abi_type(typ)]
+
+        if self.has_default_args:
+            # for functions with default args, return a dict for each possible arg count
+            result = []
+            for i in range(self.min_arg_count, self.max_arg_count + 1):
+                result.append(abi_dict.copy())
+                result[-1]["inputs"] = result[-1]["inputs"][:i]
+            return result
+        else:
+            return [abi_dict]
+
+
+def _generate_abi_type(type_definition, name=""):
+    if isinstance(type_definition, StructDefinition):
+        return {
+            "name": name,
+            "type": "tuple",
+            "components": [_generate_abi_type(v, k) for k, v in type_definition.members.items()],
+        }
+    return {"name": name, "type": type_definition.canonical_type}
+
+
+def _generate_method_id(name: str, canonical_types: List[str]) -> Dict[str, int]:
+    function_sig = f"{name}({','.join(canonical_types)})"
+    selector = keccak256(function_sig.encode())[:4].hex()
+    return {function_sig: int(selector, 16)}
