@@ -256,7 +256,7 @@ class Slice:
             ReturnType = OldStringType
 
         # Node representing the position of the output in memory
-        np = context.new_placeholder(ReturnType(maxlen=sub_typ_maxlen + 32))
+        np = context.new_internal_variable(ReturnType(maxlen=sub_typ_maxlen + 32))
         placeholder_node = LLLnode.from_list(np, typ=sub.typ, location="memory")
         placeholder_plus_32_node = LLLnode.from_list(np + 32, typ=sub.typ, location="memory")
         # Copies over bytearray data
@@ -412,7 +412,7 @@ class Concat:
             [arg.typ.maxlen if isinstance(arg.typ, ByteArrayLike) else 32 for arg in args]
         )
         # Node representing the position of the output in memory
-        placeholder = context.new_placeholder(ReturnType(total_maxlen))
+        placeholder = context.new_internal_variable(ReturnType(total_maxlen))
         # Object representing the output
         seq = []
         # For each argument we are concatenating...
@@ -573,7 +573,7 @@ class Sha256(_SimpleBuiltinFunction):
         # bytearay-like input
         if sub.location == "storage":
             # Copy storage to memory
-            placeholder = context.new_placeholder(sub.typ)
+            placeholder = context.new_internal_variable(sub.typ)
             placeholder_node = LLLnode.from_list(placeholder, typ=sub.typ, location="memory")
             copier = make_byte_array_copier(
                 placeholder_node, LLLnode.from_list("_sub", typ=sub.typ, location=sub.location),
@@ -681,7 +681,9 @@ class ECRecover(_SimpleBuiltinFunction):
     @validate_inputs
     def build_LLL(self, expr, args, kwargs, context):
         placeholder_node = LLLnode.from_list(
-            context.new_placeholder(ByteArrayType(128)), typ=ByteArrayType(128), location="memory"
+            context.new_internal_variable(ByteArrayType(128)),
+            typ=ByteArrayType(128),
+            location="memory",
         )
         return LLLnode.from_list(
             [
@@ -725,7 +727,9 @@ class ECAdd(_SimpleBuiltinFunction):
     @validate_inputs
     def build_LLL(self, expr, args, kwargs, context):
         placeholder_node = LLLnode.from_list(
-            context.new_placeholder(ByteArrayType(128)), typ=ByteArrayType(128), location="memory"
+            context.new_internal_variable(ByteArrayType(128)),
+            typ=ByteArrayType(128),
+            location="memory",
         )
         pos = getpos(expr)
         o = LLLnode.from_list(
@@ -754,7 +758,9 @@ class ECMul(_SimpleBuiltinFunction):
     @validate_inputs
     def build_LLL(self, expr, args, kwargs, context):
         placeholder_node = LLLnode.from_list(
-            context.new_placeholder(ByteArrayType(128)), typ=ByteArrayType(128), location="memory"
+            context.new_internal_variable(ByteArrayType(128)),
+            typ=ByteArrayType(128),
+            location="memory",
         )
         pos = getpos(expr)
         o = LLLnode.from_list(
@@ -1033,10 +1039,10 @@ class RawCall(_SimpleBuiltinFunction):
                 " use `is_static_call=True` to perform this action",
                 expr,
             )
-        placeholder = context.new_placeholder(data.typ)
+        placeholder = context.new_internal_variable(data.typ)
         placeholder_node = LLLnode.from_list(placeholder, typ=data.typ, location="memory")
         copier = make_byte_array_copier(placeholder_node, data, pos=getpos(expr))
-        output_placeholder = context.new_placeholder(ByteArrayType(outsize))
+        output_placeholder = context.new_internal_variable(ByteArrayType(outsize))
         output_node = LLLnode.from_list(
             output_placeholder, typ=ByteArrayType(outsize), location="memory",
         )
@@ -1151,7 +1157,7 @@ class RawLog:
                 raise TypeMismatch("Expecting a bytes32 argument as topic", elt)
             topics.append(arg)
         if args[1].typ == BaseType("bytes32"):
-            placeholder = context.new_placeholder(BaseType("bytes32"))
+            placeholder = context.new_internal_variable(BaseType("bytes32"))
             return LLLnode.from_list(
                 [
                     "seq",
@@ -1172,7 +1178,7 @@ class RawLog:
                 typ=None,
                 pos=getpos(expr),
             )
-        placeholder = context.new_placeholder(args[1].typ)
+        placeholder = context.new_internal_variable(args[1].typ)
         placeholder_node = LLLnode.from_list(placeholder, typ=args[1].typ, location="memory")
         copier = make_byte_array_copier(
             placeholder_node,
@@ -1325,15 +1331,25 @@ class Shift(_SimpleBuiltinFunction):
             shift_abs = ["sub", 0, "_s"]
 
         if version_check(begin="constantinople"):
-            left_shift = ["shl", "_s", "_v"]
-            right_shift = ["shr", shift_abs, "_v"]
+            if args[1].typ.is_literal:
+                # optimization when SHL/SHR instructions are available shift distance is a literal
+                value = args[1].value
+                if value >= 0:
+                    lll_node = ["shl", value, args[0]]
+                else:
+                    lll_node = ["shr", abs(value), args[0]]
+                return LLLnode.from_list(lll_node, typ=BaseType("uint256"), pos=getpos(expr))
+            else:
+                left_shift = ["shl", "_s", args[0]]
+                right_shift = ["shr", shift_abs, args[0]]
+
         else:
             # If second argument is positive, left-shift so multiply by a power of two
             # If it is negative, divide by a power of two
             # node that if the abs of the second argument >= 256, then in the EVM
             # 2**(second arg) = 0, and multiplying OR dividing by 0 gives 0
-            left_shift = ["mul", "_v", ["exp", 2, "_s"]]
-            right_shift = ["div", "_v", ["exp", 2, shift_abs]]
+            left_shift = ["mul", args[0], ["exp", 2, "_s"]]
+            right_shift = ["div", args[0], ["exp", 2, shift_abs]]
 
         if not args[1].typ.is_literal:
             node_list = ["if", ["slt", "_s", 0], right_shift, left_shift]
@@ -1343,9 +1359,7 @@ class Shift(_SimpleBuiltinFunction):
             node_list = right_shift
 
         return LLLnode.from_list(
-            ["with", "_v", args[0], ["with", "_s", args[1], node_list]],
-            typ=BaseType("uint256"),
-            pos=getpos(expr),
+            ["with", "_s", args[1], node_list], typ=BaseType("uint256"), pos=getpos(expr),
         )
 
 
@@ -1412,54 +1426,53 @@ class PowMod256(_SimpleBuiltinFunction):
 
 
 def get_create_forwarder_to_bytecode():
-    from vyper.compile_lll import assembly_to_evm, num_to_bytearray
+    from vyper.compile_lll import assembly_to_evm
 
-    code_a = [
+    loader_asm = [
         "PUSH1",
-        0x33,
+        0x2D,
+        "RETURNDATASIZE",
+        "DUP2",
         "PUSH1",
-        0x0C,
-        "PUSH1",
-        0x00,
+        0x09,
+        "RETURNDATASIZE",
         "CODECOPY",
-        "PUSH1",
-        0x33,
-        "PUSH1",
-        0x00,
         "RETURN",
+    ]
+    forwarder_pre_asm = [
         "CALLDATASIZE",
-        "PUSH1",
-        0x00,
-        "PUSH1",
-        0x00,
+        "RETURNDATASIZE",
+        "RETURNDATASIZE",
         "CALLDATACOPY",
-        "PUSH2",
-        num_to_bytearray(0x1000),
-        "PUSH1",
-        0x00,
+        "RETURNDATASIZE",
+        "RETURNDATASIZE",
+        "RETURNDATASIZE",
         "CALLDATASIZE",
-        "PUSH1",
-        0x00,
+        "RETURNDATASIZE",
         "PUSH20",  # [address to delegate to]
     ]
-    code_b = [
+    forwarder_post_asm = [
         "GAS",
         "DELEGATECALL",
-        "PUSH1",
-        0x2C,  # jumpdest of whole program.
-        "JUMPI",
-        "PUSH1",
-        0x0,
+        "RETURNDATASIZE",
+        "DUP3",
         "DUP1",
+        "RETURNDATACOPY",
+        "SWAP1",
+        "RETURNDATASIZE",
+        "SWAP2",
+        "PUSH1",
+        0x2B,  # jumpdest of whole program.
+        "JUMPI",
         "REVERT",
         "JUMPDEST",
-        "PUSH2",
-        num_to_bytearray(0x1000),
-        "PUSH1",
-        0x00,
         "RETURN",
     ]
-    return assembly_to_evm(code_a)[0] + (b"\x00" * 20) + assembly_to_evm(code_b)[0]
+    return (
+        assembly_to_evm(loader_asm)[0],
+        assembly_to_evm(forwarder_pre_asm)[0],
+        assembly_to_evm(forwarder_post_asm)[0],
+    )
 
 
 class CreateForwarderTo(_SimpleBuiltinFunction):
@@ -1476,19 +1489,30 @@ class CreateForwarderTo(_SimpleBuiltinFunction):
             raise StateAccessViolation(
                 f"Cannot make calls from {context.pp_constancy()}", expr,
             )
-        placeholder = context.new_placeholder(ByteArrayType(96))
+        placeholder = context.new_internal_variable(ByteArrayType(96))
 
-        kode = get_create_forwarder_to_bytecode()
-        high = bytes_to_int(kode[:32])
-        low = bytes_to_int((kode + b"\x00" * 32)[47:79])
+        loader_evm, forwarder_pre_evm, forwarder_post_evm = get_create_forwarder_to_bytecode()
+        # Adjust to 32-byte boundaries
+        preamble_length = len(loader_evm) + len(forwarder_pre_evm)
+        forwarder_preamble = bytes_to_int(
+            loader_evm + forwarder_pre_evm + b"\x00" * (32 - preamble_length)
+        )
+        forwarder_post = bytes_to_int(forwarder_post_evm + b"\x00" * (32 - len(forwarder_post_evm)))
+
+        if args[0].typ.is_literal:
+            target_address = args[0].value * 2 ** 96
+        elif version_check(begin="constantinople"):
+            target_address = ["shl", 96, args[0]]
+        else:
+            target_address = ["mul", args[0], 2 ** 96]
 
         return LLLnode.from_list(
             [
                 "seq",
-                ["mstore", placeholder, high],
-                ["mstore", ["add", placeholder, 27], ["mul", args[0], 2 ** 96]],
-                ["mstore", ["add", placeholder, 47], low],
-                ["clamp_nonzero", ["create", value, placeholder, 96]],
+                ["mstore", placeholder, forwarder_preamble],
+                ["mstore", ["add", placeholder, preamble_length], target_address],
+                ["mstore", ["add", placeholder, preamble_length + 20], forwarder_post],
+                ["create", value, placeholder, preamble_length + 20 + len(forwarder_post_evm)],
             ],
             typ=BaseType("address"),
             pos=getpos(expr),
@@ -1612,7 +1636,7 @@ else:
             new_var_pos = arg.args[0]
         # Other locations need to be copied.
         else:
-            new_var_pos = context.new_placeholder(x_type)
+            new_var_pos = context.new_internal_variable(x_type)
             placeholder_copy = ["mstore", new_var_pos, arg]
         # Create input variables.
         variables = {"x": VariableRecord(name="x", pos=new_var_pos, typ=x_type, mutable=False)}
