@@ -1,4 +1,6 @@
-from vyper.parser.context import Constancy, Context
+# can't use from [module] import [object] because it breaks mocks in testing
+from vyper.parser import context as ctx
+from vyper.parser.context import Constancy
 from vyper.parser.function_definitions.parse_external_function import (  # NOTE black/isort conflict
     parse_external_function,
 )
@@ -20,7 +22,7 @@ def is_default_func(code):
     return code.name == "__default__"
 
 
-def parse_function(code, sigs, origcode, global_ctx, is_contract_payable, _vars=None):
+def parse_function(code, sigs, global_ctx, check_nonpayable, _vars=None):
     """
     Parses a function and produces LLL code for the function, includes:
         - Signature method if statement
@@ -37,7 +39,7 @@ def parse_function(code, sigs, origcode, global_ctx, is_contract_payable, _vars=
 
     # Create a local (per function) context.
     memory_allocator = MemoryAllocator()
-    context = Context(
+    context = ctx.Context(
         vars=_vars,
         global_ctx=global_ctx,
         sigs=sigs,
@@ -45,7 +47,6 @@ def parse_function(code, sigs, origcode, global_ctx, is_contract_payable, _vars=
         return_type=sig.output_type,
         constancy=Constancy.Constant if sig.mutability in ("view", "pure") else Constancy.Mutable,
         is_payable=sig.mutability == "payable",
-        origcode=origcode,
         is_internal=sig.internal,
         method_id=sig.method_id,
         sig=sig,
@@ -55,10 +56,10 @@ def parse_function(code, sigs, origcode, global_ctx, is_contract_payable, _vars=
         o = parse_internal_function(code=code, sig=sig, context=context,)
     else:
         o = parse_external_function(
-            code=code, sig=sig, context=context, is_contract_payable=is_contract_payable
+            code=code, sig=sig, context=context, check_nonpayable=check_nonpayable
         )
 
     o.context = context
-    o.total_gas = o.gas + calc_mem_gas(o.context.memory_allocator.get_next_memory_position())
+    o.total_gas = o.gas + calc_mem_gas(o.context.memory_allocator.size_of_mem)
     o.func_name = sig.name
     return o
