@@ -24,14 +24,7 @@ from vyper.types import (
     has_dynamic_data,
     is_base_type,
 )
-from vyper.types.types import InterfaceType
-from vyper.utils import (
-    DECIMAL_DIVISOR,
-    GAS_IDENTITY,
-    GAS_IDENTITYWORD,
-    MemoryPositions,
-    SizeLimits,
-)
+from vyper.utils import GAS_IDENTITY, GAS_IDENTITYWORD, MemoryPositions
 
 getcontext().prec = 78  # MAX_UINT256 < 1e78
 
@@ -174,6 +167,11 @@ def make_byte_slice_copier(destination, source, length, max_length, pos=None):
         loader = ["mload", ["add", "_pos", ["mul", 32, ["mload", MemoryPositions.FREE_LOOP_INDEX]]]]
     elif source.location == "storage":
         loader = ["sload", ["add", "_pos", ["mload", MemoryPositions.FREE_LOOP_INDEX]]]
+    elif source.location == "calldata":
+        loader = [
+            "calldataload",
+            ["add", "_pos", ["mul", 32, ["mload", MemoryPositions.FREE_LOOP_INDEX]]],
+        ]
     else:
         raise CompilerPanic(f"Unsupported location: {source.location}")
     # Where to paste it?
@@ -271,19 +269,6 @@ def getpos(node):
     )
 
 
-def set_offsets(node, pos):
-    # TODO replace this with a visitor pattern
-    for field in node.get_fields():
-        item = getattr(node, field, None)
-        if isinstance(item, vy_ast.VyperNode):
-            set_offsets(item, pos)
-        elif isinstance(item, list):
-            for i in item:
-                if isinstance(i, vy_ast.VyperNode):
-                    set_offsets(i, pos)
-    node.lineno, node.col_offset, node.end_lineno, node.end_col_offset = pos
-
-
 # Take a value representing a memory or storage location, and descend down to
 # an element or member variable
 @type_check_wrapper
@@ -343,12 +328,14 @@ def add_variable_offset(parent, key, pos, array_bounds_check=True):
                         ]
                     )
                 else:
-                    sub = LLLnode.from_list(
-                        ["sha3", ["add", key.args[0].value, 32], ["mload", key.args[0].value]]
-                    )
+                    value = key.args[0].value
+                    if value == "add":
+                        # special case, key is a bytes array within a tuple/struct
+                        value = key.args[0]
+                    sub = LLLnode.from_list(["sha3", ["add", value, 32], key])
         else:
             subtype = typ.valuetype
-            sub = base_type_conversion(key, key.typ, typ.keytype, pos=pos)
+            sub = unwrap_location(key)
 
         if sub is not None and location == "storage":
             return LLLnode.from_list(["sha3_64", parent, sub], typ=subtype, location="storage")
@@ -384,42 +371,6 @@ def add_variable_offset(parent, key, pos, array_bounds_check=True):
             )
 
 
-# Convert from one base type to another
-@type_check_wrapper
-def base_type_conversion(orig, frm, to, pos, in_function_call=False):
-    orig = unwrap_location(orig)
-
-    # do the base type check so we can use BaseType attributes
-    if not isinstance(frm, BaseType) or not isinstance(to, BaseType):
-        return
-
-    if getattr(frm, "is_literal", False):
-        for typ in (frm.typ, to.typ):
-            if typ in ("int128", "uint256") and not SizeLimits.in_bounds(typ, orig.value):
-                return
-
-    is_decimal_int128_conversion = frm.typ == "int128" and to.typ == "decimal"
-    is_same_type = frm.typ == to.typ
-    is_literal_conversion = frm.is_literal and (frm.typ, to.typ) == ("int128", "uint256")
-    is_address_conversion = isinstance(frm, InterfaceType) and to.typ == "address"
-    if not (
-        is_same_type
-        or is_literal_conversion
-        or is_address_conversion
-        or is_decimal_int128_conversion
-    ):
-        return
-
-    # handle None value inserted by `empty()`
-    if orig.value is None:
-        return LLLnode.from_list(0, typ=to)
-
-    if is_decimal_int128_conversion:
-        return LLLnode.from_list(["mul", orig, DECIMAL_DIVISOR], typ=BaseType("decimal"),)
-
-    return LLLnode(orig.value, orig.args, typ=to, add_gas_estimate=orig.add_gas_estimate)
-
-
 # Unwrap location
 def unwrap_location(orig):
     if orig.location == "memory":
@@ -429,6 +380,9 @@ def unwrap_location(orig):
     elif orig.location == "calldata":
         return LLLnode.from_list(["calldataload", orig], typ=orig.typ)
     else:
+        # handle None value inserted by `empty`
+        if orig.value is None:
+            return LLLnode.from_list(0, typ=orig.typ)
         return orig
 
 
@@ -438,14 +392,13 @@ def pack_arguments(signature, args, context, stmt_expr, is_external_call):
     pos = getpos(stmt_expr)
     setters = []
     staticarray_offset = 0
-    needpos = False
 
     maxlen = sum([get_size_of_type(arg.typ) for arg in signature.args]) * 32
     if is_external_call:
         maxlen += 32
 
     placeholder_typ = ByteArrayType(maxlen=maxlen)
-    placeholder = context.new_placeholder(placeholder_typ)
+    placeholder = context.new_internal_variable(placeholder_typ)
     if is_external_call:
         setters.append(["mstore", placeholder, signature.method_id])
         placeholder += 32
@@ -453,6 +406,10 @@ def pack_arguments(signature, args, context, stmt_expr, is_external_call):
     if len(signature.args) != len(args):
         return
 
+    # check for dynamic-length types
+    dynamic_remaining = len([i for i in signature.args if isinstance(i.typ, ByteArrayLike)])
+    needpos = bool(dynamic_remaining)
+
     for i, (arg, typ) in enumerate(zip(args, [arg.typ for arg in signature.args])):
         if isinstance(typ, BaseType):
             setters.append(
@@ -466,26 +423,34 @@ def pack_arguments(signature, args, context, stmt_expr, is_external_call):
             )
 
         elif isinstance(typ, ByteArrayLike):
+            dynamic_remaining -= 1
             setters.append(["mstore", placeholder + staticarray_offset + i * 32, "_poz"])
             arg_copy = LLLnode.from_list("_s", typ=arg.typ, location=arg.location)
             target = LLLnode.from_list(["add", placeholder, "_poz"], typ=typ, location="memory",)
-            setters.append(
-                [
-                    "with",
-                    "_s",
-                    arg,
+            pos_setter = "pass"
+
+            # if `arg.value` is None, this is a call to `empty()`
+            # if `arg.typ.maxlen` is 0, this is a literal "" or b""
+            if arg.value is None or arg.typ.maxlen == 0:
+                if dynamic_remaining:
+                    # only adjust the dynamic pointer if this is not the last dynamic type
+                    pos_setter = ["set", "_poz", ["add", "_poz", 64]]
+                setters.append(["seq", mzero(target, 64), pos_setter])
+            else:
+                if dynamic_remaining:
+                    pos_setter = [
+                        "set",
+                        "_poz",
+                        ["add", 32, ["ceil32", ["add", "_poz", get_length(arg_copy)]]],
+                    ]
+                setters.append(
                     [
-                        "seq",
-                        make_byte_array_copier(target, arg_copy, pos),
-                        [
-                            "set",
-                            "_poz",
-                            ["add", 32, ["ceil32", ["add", "_poz", get_length(arg_copy)]]],
-                        ],
-                    ],
-                ]
-            )
-            needpos = True
+                        "with",
+                        "_s",
+                        arg,
+                        ["seq", make_byte_array_copier(target, arg_copy, pos), pos_setter],
+                    ]
+                )
 
         elif isinstance(typ, (StructType, ListType)):
             if has_dynamic_data(typ):
@@ -543,9 +508,7 @@ def _make_array_index_setter(target, target_token, pos, location, offset):
 def make_setter(left, right, location, pos, in_function_call=False):
     # Basic types
     if isinstance(left.typ, BaseType):
-        right = base_type_conversion(
-            right, right.typ, left.typ, pos, in_function_call=in_function_call,
-        )
+        right = unwrap_location(right)
         if location == "storage":
             return LLLnode.from_list(["sstore", left, right], typ=None)
         elif location == "memory":
@@ -757,15 +720,6 @@ def _check_return_body(node, node_list):
             )
 
 
-def check_unmatched_return(fn_node):
-    if fn_node.returns and not _return_check(fn_node.body):
-        raise StructureException(
-            f'Missing or Unmatched return statements in function "{fn_node.name}". '
-            "All control flow statements (like if) need balanced return statements.",
-            fn_node,
-        )
-
-
 def _return_check(node):
     if is_return_from_function(node):
         return True
